/*
 * Title:        GridSim Toolkit
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation 
 *               of Parallel and Distributed Systems such as Clusters and Grids
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 *
 * $Id: GridInformationService.java,v 1.11 2003/06/30 05:21:01 anthony Exp $
 */

package gridsim;

import java.util.LinkedList;
import eduni.simjava.Sim_event;

/*--------------------------------------------------------------------------- */
// Classes related to GridSim Resources: GIS, Local Resource Schedulers
/*--------------------------------------------------------------------------- */

/**
 * A Grid Information Service (GIS) is a GridSim entity that provides Grid 
 * resource registration, indexing and discovery services. The Grid resources
 * register their readiness to process Gridlets by registering themselves with
 * this entity. 
 * <p>
 * Other entities such as the resource broker can contact
 * this class for resource discovery service, which returns a list of
 * registered resource entities and their contact address. For example,
 * scheduling entities use this service for resource discovery.
 * <p>
 * In summary, it acts like a yellow page service.
 *
 * @author       Manzur Murshed and Rajkumar Buyya
 * @version      2.1, June 2003
 * @invariant $none
 */
public class GridInformationService extends GridSim
{
    /**
     * Allocates a new GridInformationService object with networked I/O ports
     * @param name       the name to be associated with this entity (as 
     *                   required by Sim_entity class from simjava package)
     * @param baud_rate  communication speed
     * @throws Exception This happens when creating this entity before 
     *                   initializing GridSim package or this entity name is
     *                   <tt>null</tt> or empty
     * @see gridsim.GridSim#init(int, Calendar, boolean, String[], String[],
     *          String)
     * @see eduni.simjava.Sim_entity
     * @pre name != null
     * @pre baud_rate > 0.0
     * @post $none
     */
    public GridInformationService(String name, double baud_rate) 
                    throws Exception
    {
        super(name, baud_rate);
    }

    /**
     * A method that gets one event at a time, and serves it based
     * on its request.
     * <p>
     * The services available to other GridSim entities are:
     * <ul>
     *      <li> GridSimTags.REGISTER_RESOURCE </li>
     *      <li> GridSimTags.RESOURCE_LIST </li>
     * </ul>
     * @pre $none
     * @post $none
     */
    public void body()
    {
        LinkedList ResourceList = new LinkedList();

        // Process events
        Sim_event ev = new Sim_event();
        for ( sim_get_next(ev); ev.get_tag() != GridSimTags.END_OF_SIMULATION; 
                sim_get_next(ev) )
        {
            switch ( ev.get_tag() )
            {
                // A resource is requesting to register.
                case GridSimTags.REGISTER_RESOURCE: 
                    ResourceList.add( (Integer) ev.get_data() );
                    break;

                // A Broker is requesting for a list of resources.
                case GridSimTags.RESOURCE_LIST: 

                    // Get ID of Broker that send this event
                    int requester_id = ( (Integer) ev.get_data() ).intValue();
                    
                    // Send the event or message
                    super.send( super.output, GridSimTags.SCHEDULE_NOW, 
                            ev.get_tag(), new IO_data(ResourceList, 
                                4*ResourceList.size(), requester_id) );
                    break;

                default:
                    System.out.println("GridInformationSevice.body() : " +
                            "Unable to handle request from GridSimTags " +
                            "with constant number " + ev.get_tag() );
                    break;
            }
        }
        
        // after finish the simulation, disconnect I/O ports
        super.terminateIOEntities();
    }
    
} // end class

