/*
 * Title:        GridSim Toolkit
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation 
 *               of Parallel and Distributed Systems such as Clusters and Grids
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 *
 * $Id: Gridlet.java,v 1.8 2003/06/30 05:21:01 anthony Exp $
 */

package gridsim;

import java.util.*;
import java.io.*;
import eduni.simjava.*;
import gridsim.*;

/**
 * GridSim Gridlet class acts as job package that contains job length in
 * Millions Instructions (MI), the length of input and output data in bytes,
 * execution start and end time, and the originator of the job. 
 * <p>
 * Individual users model their application by creating Gridlets for 
 * processing them
 * on Grid resources assigned by scheduling entities (resource brokers).
 *
 * @author       Manzur Murshed and Rajkumar Buyya
 * @version      2.1, June 2003
 * @invariant $none
 */
public class Gridlet 
{
    // the User or Broker ID. It is advisable that broker set this ID
    // with its own ID, so that GridResource returns to it after the execution
    private int userID_;  
       
    private int gridletID_;          // Gridlet ID
    private double gridletLength_;   // in MI
    private long gridletFileSize_;   // in byte = program + input data size
    private long gridletOutputSize_; // in byte
    private int resourceID_;         // the resource on which Gridlet executed
    private double costPerSec_;      // Agreed cost (G$) of Resource/CPU-sec
    
    // Gridlet SubmitTime is required since Gridlet execution may not start 
    // immediately. This particularly important when we use Space Shared 
    // resource
    private double submissionTime_;  // submission Time to a resource
    
    // Start time of Gridlet execution, 
    // if Time Shared it is same as Submission Time.
    private double execStartTime_;   
    
    private double wallClockTime_;   // Gridlets in seconds
    private double actualCPUTime_;   // in seconds
    private int status_;             // status of a Gridlet

    /** The Gridlet has been created and added to the GridletList object */
    public static final int CREATED = 0; 

    /** The Gridlet has been assigned to a GridResource object as planned */
    public static final int READY = 1;   

    /** The Gridlet has moved to a Grid node */
    public static final int QUEUED = 2;  
    
    /** The Gridlet is in execution in a Grid node */
    public static final int INEXEC = 3;  
    
    /** The Gridlet has been executed successfully */
    public static final int SUCCESS = 4; 
    
    /** The Gridlet is failed */
    public static final int FAILED = 5;  


    /**
     * Allocates a new Gridlet object
     * @param gridletID            the ID of a Gridlet
     * @param gridletLength        the length or size of a Gridlet
     * @param gridletFileSize      the file size of a Gridlet 
     * @param gridletOutputSize    the output size of a Gridlet
     * @pre gridletID >= 0
     * @pre gridletLength >= 0.0
     * @pre gridletFileSize >= 1
     * @pre gridletOutputSize >= 1
     * @post $none
     */
    public Gridlet(int gridletID, double gridletLength,
                long gridletFileSize, long gridletOutputSize) 
    {
        // to be set by the Broker or other scheduler
        this.userID_ = -1;              
        this.resourceID_ = 0;

        this.gridletID_ = gridletID;
        this.gridletLength_ = gridletLength;
        this.status_ = CREATED;
        
        // Input and Output size should be at least 1 byte.
        this.gridletFileSize_ = Math.max(1, gridletFileSize);
        this.gridletOutputSize_ = Math.max(1, gridletOutputSize);

        this.actualCPUTime_ = 0.0;
        this.costPerSec_ = 0.0;
        this.execStartTime_ = 0.0;
        this.submissionTime_ = 0.0;
        this.wallClockTime_ = 0.0;
    }

    /**
     * Gets the Gridlet ID
     * @return the Gridlet ID
     * @deprecated As of GridSim 2.1, replaced by {@link #getGridletID()}
     * @pre $none
     * @post $result >= 0
     */
    public int GetGridletID() {
        return this.getGridletID();
    }

    /**
     * Gets the Gridlet ID
     * @return the Gridlet ID
     * @pre $none
     * @post $result >= 0
     */
    public int getGridletID() {
        return gridletID_;
    }

    /**
     * Sets the user ID
     * @param id  the user ID
     * @deprecated As of GridSim 2.1, replaced by {@link #setUserID(int)}
     * @pre id >= 0
     * @post $none
     */
    public void SetUserID(int id) {
        this.setUserID(id);
    }

    /**
     * Sets the user ID
     * @param id  the user ID
     * @pre id >= 0
     * @post $none
     */
    public void setUserID(int id) {
        userID_ = id;
    }

    /**
     * Gets the user ID
     * @return the user ID or <tt>-1</tt> if the user ID has not been set before
     * @deprecated As of GridSim 2.1, replaced by {@link #getUserID()}
     * @pre $none
     * @post $result >= -1
     */
    public int GetUserID() {
        return this.getUserID();
    }

    /**
     * Gets the user ID
     * @return the user ID or <tt>-1</tt> if the user ID has not been set before
     * @pre $none
     * @post $result >= -1
     */
    public int getUserID() {
        return userID_;
    }

    /**
     * Gets the resource ID
     * @return the resource ID
     * @deprecated As of GridSim 2.1, replaced by {@link #getResourceID()}
     * @pre $none
     * @post $result >= 0
     */
    public int GetResourceID() {
        return this.getResourceID(); 
    }

    /**
     * Gets the resource ID
     * @return the resource ID
     * @pre $none
     * @post $result >= 0
     */
    public int getResourceID() {
        return resourceID_;
    }

    /**
     * Gets the file size of a Gridlet
     * @return the Gridlet file size
     * @deprecated As of GridSim 2.1, replaced by {@link #getGridletFileSize()}
     * @pre $none
     * @post $result >= 1
     */
    public long GetGridletFileSize() {
        return this.getGridletFileSize();
    }

    /**
     * Gets the file size of a Gridlet
     * @return the Gridlet file size
     * @pre $none
     * @post $result >= 1
     */
    public long getGridletFileSize() {
        return gridletFileSize_;
    }

    /**
     * Gets the output size of a Gridlet
     * @return the Gridlet output size
     * @deprecated As of GridSim 2.1, replaced by 
     *             {@link #getGridletOutputSize()}
     * @pre $none
     * @post $result >= 1
     */
    public long GetGridletOutputSize() {
        return this.getGridletOutputSize();
    }

    /**
     * Gets the output size of a Gridlet
     * @return the Gridlet output size
     * @pre $none
     * @post $result >= 1
     */
    public long getGridletOutputSize() {
        return gridletOutputSize_;
    }

    /**
     * Sets the resource parameters
     * @param resourceID   the resource ID
     * @param costPerSec   the costs associated with this resource
     * @deprecated As of GridSim 2.1, replaced by
     *             {@link #setResourceParameter(int, double)}
     * @pre resourceID >= 0
     * @pre costPerSec > 0.0
     * @post $none 
     */
    public void SetResParam(int resourceID, double costPerSec) {
        this.setResourceParameter(resourceID, costPerSec);
    }

    /**
     * Sets the resource parameters
     * @param resourceID   the resource ID
     * @param costPerSec   the costs associated with this resource
     * @pre resourceID >= 0
     * @pre costPerSec > 0.0
     * @post $none 
     */
    public void setResourceParameter(int resourceID, double costPerSec)
    {
        this.resourceID_ = resourceID;
        this.costPerSec_ = costPerSec;
    }

    /**
     * Sets the submission time
     * @param clockTime     the submission time
     * @deprecated As of GridSim 2.1, replaced by 
     *             {@link #setSubmissionTime(double)}
     * @pre clockTime >= 0.0
     * @post $none
     */
    public void SetSubmissionTime(double clockTime) {
        this.setSubmissionTime(clockTime);
    }

    /**
     * Sets the submission time
     * @param clockTime     the submission time
     * @pre clockTime >= 0.0
     * @post $none
     */
    public void setSubmissionTime(double clockTime) {
        submissionTime_ = clockTime;
    }

    /**
     * Gets the submission time
     * @return the submission time
     * @pre $none
     * @post $result >= 0.0
     */
    public double getSubmissionTime() {
        return submissionTime_;
    }

    /**
     * Sets the execution start time
     * @param clockTime     the execution start time
     * @deprecated As of GridSim 2.1, replaced by 
     *             {@link #setExecStartTime(double)}
     * @pre clockTime >= 0.0
     * @post $none
     */
    public void SetExecStartTime(double clockTime) {
        this.setExecStartTime(clockTime);
    }


    /**
     * Sets the execution start time
     * @param clockTime     the execution start time
     * @pre clockTime >= 0.0
     * @post $none
     */
    public void setExecStartTime(double clockTime) {
        execStartTime_ = clockTime;
    }

    /**
     * Gets the execution start time
     * @return the execution start time
     * @pre $none
     * @post $result >= 0.0
     */
    public double getExecStartTime() {
        return execStartTime_;
    }

    /**
     * Sets execution parameters. These two parameters are set by 
     * the GridResource after execution.
     * @param wallClockTime    the virtual real time
     * @param actualCPUTime    the time based on CPU time
     * @deprecated As of GridSim 2.1, replaced by 
     *             {@link #setExecParam(double, double)}
     * @pre wallClockTime >= 0.0
     * @pre actualCPUTime >= 0.0
     * @post $none
     */
    public void SetExecParam(double wallClockTime, double actualCPUTime) {
        this.setExecParam(wallClockTime, actualCPUTime);
    }

    /**
     * Sets execution parameters. These two parameters are set by 
     * the GridResource after execution.
     * @param wallClockTime    the virtual real time
     * @param actualCPUTime    the time based on CPU time
     * @pre wallClockTime >= 0.0
     * @pre actualCPUTime >= 0.0
     * @post $none
     */
    public void setExecParam(double wallClockTime, double actualCPUTime) 
    {
        wallClockTime_ = wallClockTime;
        actualCPUTime_ = actualCPUTime;
    }

    /**
     * Sets the status code of a Gridlet 
     * @param status    the status code of a Gridlet
     * @deprecated As of GridSim 2.1, replaced by {@link #setGridletStatus(int)}
     * @throws Exception Invalid Gridlet status
     * @pre status >= 0 && status <= 5
     * @post $none
     */
    public void SetStatus(int status) throws Exception {
        this.setGridletStatus(status);
    }

    /**
     * Sets the status code of a Gridlet 
     * @param status    the status code of a Gridlet
     * @throws Exception Invalid Gridlet status
     * @pre status >= 0 && status <= 5
     * @post $none
     */
    public void setGridletStatus(int status) throws Exception
    {
        if (status < CREATED || status > FAILED) {
            throw new Exception("Gridlet.setGridletStatus() : Error - " +
                    "Invalid integer range for Gridlet status.");
        }
        
        this.status_ = status;
    }

    /**
     * Gets the status code of a Gridlet
     * @return the status code of a Gridlet
     * @deprecated As of GridSim 2.1, replaced by {@link #getGridletStatus()}
     * @pre $none
     * @post $result >= 0
     */
    public int GetStatus() {
        return this.getGridletStatus();
    }

    /**
     * Gets the status code of a Gridlet
     * @return the status code of a Gridlet
     * @pre $none
     * @post $result >= 0
     */
    public int getGridletStatus() {
        return status_;
    }

    /**
     * Gets the string representation of the current Gridlet status code
     * @return the Gridlet status code as a string or <tt>null</tt> if the
     *         status code is unknown
     * @deprecated As of GridSim 2.1, replaced by 
     *             {@link #getGridletStatusString()}
     * @pre $none
     * @post $none
     */
    public String GetStatusString() {
        return this.getGridletStatusString();
    }

    /**
     * Gets the string representation of the current Gridlet status code
     * @return the Gridlet status code as a string or <tt>null</tt> if the
     *         status code is unknown
     * @pre $none
     * @post $none
     */
    public String getGridletStatusString()
    {
        String statusString = null;
        switch ( status_ )
        {
            case CREATED:
                statusString = "Created";
                break;
                
            case READY:
                statusString = "Ready";
                break;
                
            case INEXEC:
                statusString = "InExec";
                break;
                
            case SUCCESS:
                statusString = "Success";
                break;
                
            case FAILED:
                statusString = "Failed";
                break;

            default:
                break;
        }
        
        return statusString;
    }

    /**
     * Gets the length of a Gridlet
     * @return the length of a Gridlet
     * @deprecated As of GridSim 2.1, replaced by {@link #getGridletLength()}
     * @pre $none
     * @post $result >= 0.0
     */
    public double GetGridletLength() {
        return this.getGridletLength();
    }

    /**
     * Gets the length of a Gridlet
     * @return the length of a Gridlet
     * @pre $none
     * @post $result >= 0.0
     */
    public double getGridletLength() {
        return gridletLength_;
    }

    /**
     * Gets the CPU time
     * @return the CPU time
     * @deprecated As of GridSim 2.1, replaced by {@link #getActualCPUTime()}
     * @pre $none
     * @post $result >= 0.0
     */
    public double GetActualCPUTime() {
        return this.getActualCPUTime();
    }

    /**
     * Gets the CPU time
     * @return the CPU time
     * @pre $none
     * @post $result >= 0.0
     */
    public double getActualCPUTime() {
        return actualCPUTime_;
    }

    /**
     * Gets the cost associated with a resource
     * @return the cost associated with a resource
     * @deprecated As of GridSim 2.1, replaced by {@link #getCostPerSec()}
     * @pre $none
     * @post $result >= 0.0
     */
    public double GetCostPerSec() {
        return this.getCostPerSec();
    }

    /**
     * Gets the cost associated with a resource
     * @return the cost associated with a resource
     * @pre $none
     * @post $result >= 0.0
     */
    public double getCostPerSec() {
        return costPerSec_;
    }
    
    /**
     * Gets the cost of processing Gridlet
     * @return the cost of processing Gridlet
     * @deprecated As of GridSim 2.1, replaced by {@link #getProcessingCost()}
     * @pre $none
     * @post $result >= 0.0
     */
    public double GetProcessingCost() {
        return this.getProcessingCost();
    }

    /**
     * Gets the cost of processing Gridlet
     * @return the cost of processing Gridlet
     * @pre $none
     * @post $result >= 0.0
     */
    public double getProcessingCost() {
        return getActualCPUTime() * getCostPerSec();
    }

    /**
     * Gets the actual real time
     * @return the actual real time
     * @deprecated As of GridSim 2.1, replaced by {@link #getWallClockTime()}
     * @pre $none
     * @post $result >= 0.0
     */
    public double GetWallClockTime() {
        return this.getWallClockTime();
    }

    /**
     * Gets the actual real time
     * @return the actual real time
     * @pre $none
     * @post $result >= 0.0
     */
    public double getWallClockTime() {
        return wallClockTime_;
    }

    /**
     * Gets the time to complete processing a Gridlet
     * @return the time to complete processing a Gridlet
     * @deprecated As of GridSim 2.1, replaced by {@link #getFinishTime()}
     * @pre $none
     * @post $result >= 0.0
     */
    public double GetFinishTime() {
        return this.getFinishTime();
    }

    /**
     * Gets the time to complete processing a Gridlet
     * @return the time to complete processing a Gridlet
     * @pre $none
     * @post $result >= 0.0
     */
    public double getFinishTime() {
        return execStartTime_ + wallClockTime_;
    }
    
} // end class

