/*
 * Title:        GridSim Toolkit
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation 
 *               of Parallel and Distributed Systems such as Clusters and Grids
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 *
 * $Id: MachineList.java,v 1.6 2003/06/30 05:21:02 anthony Exp $
 */

package gridsim;

import java.util.LinkedList;


/**
 * GridSim MachineList simulates a collection of machines. It is up to the
 * GridSim users to define the connectivity among the machines in a collection.
 * Therefore, this class can be instantiated to model a simple LAN to cluster
 * to WAN.
 *
 * @author       Manzur Murshed and Rajkumar Buyya
 * @version      2.1, June 2003
 * @invariant $none
 */
public class MachineList extends LinkedList 
{
    /**
     * Allocates a new MachineList object
     * @pre $none
     * @post $none
     */
    public MachineList() {
        super();
    }

    /**
     * Gets the Machine object for a particular ID
     * @param id    the machine ID
     * @return the Machine object or <tt>null</tt> if no machine exists
     * @see gridsim.Machine
     * @deprecated As of GridSim 2.1, replaced by {@link #getMachine(int)}
     * @pre id > 0
     * @post $none
     */
    public Machine GetMachine(int id) {
        return this.getMachine(id);
    }
    
    /**
     * Gets the Machine object for a particular ID
     * @param id    the machine ID
     * @return the Machine object or <tt>null</tt> if no machine exists
     * @see gridsim.Machine
     * @pre id > 0
     * @post $none
     */
    public Machine getMachine(int id)
    {
        for (int i = 0; i < super.size(); i++) 
        {
            Machine obj = (Machine) super.get(i);
            if (obj.getMachineID() == id) {
                return obj;
            }
        }
        
        return null;    // no machine with given id
    }

    /**
     * Gets the total number of PEs for all Machines
     * @return number of PEs
     * @deprecated As of GridSim 2.1, replaced by {@link #getNumPE()}
     * @pre $none
     * @post $result >= 0
     */
    public int GetNoOfPEs() {
        return this.getNumPE();
    }
    
    /**
     * Gets the total number of PEs for all Machines
     * @return number of PEs
     * @pre $none
     * @post $result >= 0
     */
    public int getNumPE() 
    {
        int totalSize = 0;
        for (int i = 0; i < super.size(); i++) 
        {
            Machine obj = (Machine) super.get(i);
            totalSize += obj.getPEList().size();
        }

        return totalSize;
    }

    /**
     * Gets a Machine with free PE 
     * @return a machine object or <tt>null</tt> if not found
     * @deprecated As of GridSim 2.1, replaced by 
     *             {@link #getMachineWithFreePE()}
     * @pre $none
     * @post $none
     */
    public Machine GetMachineWithFreePE() {
        return this.getMachineWithFreePE();
    }
    
    /**
     * Gets a Machine with free PE 
     * @return a machine object or <tt>null</tt> if not found
     * @pre $none
     * @post $none
     */
    public Machine getMachineWithFreePE()
    {
        for (int i = 0; i < super.size(); i++) 
        {
            Machine obj = (Machine) super.get(i);
            PEList myPElist = obj.getPEList();
            
            if (myPElist.getFreePEID() != -1) {
                return obj;   // a machine with Free ID is found.
            }
        }
        
        return null;    // none of the machines have free PE.
    }

    /**
     * Gets the byte size of this class
     * @return the byte size
     * @deprecated As of GridSim 2.1, replaced by {@link #getByteSize()}
     * @pre $none
     * @post $result >= 0
     */
    public int ByteSize() {
        return this.getByteSize();
    }
    
    /**
     * Gets the byte size of this class
     * @return the byte size
     * @pre $none
     * @post $result >= 0
     */
    public int getByteSize() 
    {
        int bSize = 0;
        for (int i = 0; i < super.size(); i++) 
        {
            Machine obj = (Machine) super.get(i);
            bSize += obj.getByteSize();
        }
        
        return bSize;
    }
    
} // end class

